// scott.fluix@h4ck3r.net
//
// The reason this one is crazy (multiple textures, etc) is that a common "batch"
// is 2 tris big (holding a quad with a texture on it), but there are more
// complicated ones. So, we try to abuse the power of the gpu to hide some of the
// state changes.
//
// Having said all that, this is the first effect file I've ever written. Please
// drop me a line if you can suggest improvements (or a better overall way of
// doing this).

#define MAXTEX 8
texture2D Texture0;
texture2D Texture1;
texture2D Texture2;
texture2D Texture3;
texture2D Texture4;
texture2D Texture5;
texture2D Texture6;
texture2D Texture7;
sampler Sampler0w = sampler_state { Texture = <Texture0>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler1w = sampler_state { Texture = <Texture1>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler2w = sampler_state { Texture = <Texture2>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler3w = sampler_state { Texture = <Texture3>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler4w = sampler_state { Texture = <Texture4>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler5w = sampler_state { Texture = <Texture5>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler6w = sampler_state { Texture = <Texture6>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler7w = sampler_state { Texture = <Texture7>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = WRAP; AddressV = WRAP; };
sampler Sampler0c = sampler_state { Texture = <Texture0>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler1c = sampler_state { Texture = <Texture1>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler2c = sampler_state { Texture = <Texture2>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler3c = sampler_state { Texture = <Texture3>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler4c = sampler_state { Texture = <Texture4>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler5c = sampler_state { Texture = <Texture5>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler6c = sampler_state { Texture = <Texture6>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };
sampler Sampler7c = sampler_state { Texture = <Texture7>; MipFilter = None; MagFilter = LINEAR; MinFilter = LINEAR; AddressU = CLAMP; AddressV = CLAMP; };

float4x4 WorldViewProj : WORLDVIEWPROJECTION;
float4x4 TextureMatrices[MAXTEX];
float4x4 Xforms[MAXTEX];
float4 CxformMuls[MAXTEX];
float4 CxformAdds[MAXTEX];
float4 Colours[MAXTEX];

struct VS_OUTPUT
{
	float4 position : POSITION;
	float4 colour : COLOR0;
	float4 texcoord : TEXCOORD0;
	float3 chunk : TEXCOORD1;
};

VS_OUTPUT MainVS(float2 pos : POSITION, float3 chunk : NORMAL0)
{
	VS_OUTPUT ret = (VS_OUTPUT)0;
	float4 pos4 = float4(pos, 0, 1);
	int type = (int)chunk.y;
	if (type <= -2)
	{
		int index = -type - 2;
		ret.position = mul(mul(pos4, Xforms[index]), WorldViewProj);
		ret.texcoord = float4(chunk.x, chunk.z, 0, 0);
		ret.colour = saturate(float4(1,1,1,1) * CxformMuls[index] + CxformAdds[index]);
		ret.texcoord.z = index;
		ret.chunk = float3(index, 1, chunk.z);
	}
	else
	{
		ret.position = mul(mul(pos4, Xforms[chunk.x]), WorldViewProj);
		ret.texcoord = mul(pos4, TextureMatrices[chunk.x]);
		if (type == -1)
		{
			ret.colour = saturate(Colours[chunk.x] * CxformMuls[chunk.x] + CxformAdds[chunk.x]);
		}
		else
		{
			ret.colour = saturate(float4(1,1,1,1) * CxformMuls[chunk.x] + CxformAdds[chunk.x]);
		}
		ret.texcoord.z = chunk.x;
		ret.chunk = chunk;
	}
	
	return ret;
}

float4 MainPS(VS_OUTPUT vsout) : COLOR0
{
//return tex2D(Sampler0w, vsout.texcoord).rgba;
//	return float4(1,0,1,1);
	float4 finalColour = (1,1,1,1);
	int type = (int)vsout.chunk.y;
	if (type == -1)
	{
		finalColour = vsout.colour;
	}
	else
	{
		int chunk = (int)vsout.chunk.x;
		if (type == 0)
		{
			if (chunk == 0) finalColour = tex2D(Sampler0w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 1) finalColour = tex2D(Sampler1w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 2) finalColour = tex2D(Sampler2w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 3) finalColour = tex2D(Sampler3w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 4) finalColour = tex2D(Sampler4w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 5) finalColour = tex2D(Sampler5w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 6) finalColour = tex2D(Sampler6w, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 7) finalColour = tex2D(Sampler7w, vsout.texcoord).rgba * vsout.colour;
		}
		else
		{
			if (chunk == 0) finalColour = tex2D(Sampler0c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 1) finalColour = tex2D(Sampler1c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 2) finalColour = tex2D(Sampler2c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 3) finalColour = tex2D(Sampler3c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 4) finalColour = tex2D(Sampler4c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 5) finalColour = tex2D(Sampler5c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 6) finalColour = tex2D(Sampler6c, vsout.texcoord).rgba * vsout.colour;
			else if (chunk == 7) finalColour = tex2D(Sampler7c, vsout.texcoord).rgba * vsout.colour;
		}
	}
	return finalColour;
}

technique Standard
{
    pass P0
    {
		CullMode = NONE;
		AlphaBlendEnable = true;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		BlendOp = Add;
		ZEnable = true;
		ZFunc = LessEqual;
        vertexShader = compile vs_1_1 MainVS();
        pixelShader = compile ps_2_0 MainPS();
    }
}